//#pragma warning(disable: 4078)

#include "main.h"

#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include "intro.h"

//----------------------------------------------------------------------------

static int dwStyle = WS_VISIBLE|WS_POPUP|WS_CLIPSIBLINGS|WS_CLIPCHILDREN;

static BITMAPINFO   bmi = { { sizeof(BITMAPINFOHEADER),
                            XRES, -YRES, 1, 32, BI_RGB,
                            0, 0, 0, 0, 0 }, { 0, 0, 0, 0 } };

#ifdef __cplusplus
extern "C" { int _fltused = 0; }
#else
int _fltused = 0;
#endif

static HDC  hDC;
static long buffer[XRES*YRES] = {0};

//----------------------------------------------------------------------------

static void pinta( void )
{
    StretchDIBits(hDC,0,0,XRES,YRES,0,0,XRES,YRES,buffer,&bmi,DIB_RGB_COLORS,SRCCOPY);
}

static unsigned char bmpheader[54] = {
// Header = 14 bytes
0x42, 0x4D,              //  0: signature = BM
0x00, 0x00, 0x00, 0x00,  //  2: [size]
0x00, 0x00,              //  6: reserved
0x00, 0x00,              //  8: reserved
0x36, 0x00, 0x00, 0x00,  // 10: offset to pixels = 54 = sizeof(Header) + sizeof(Info)

// Info = 40 bytes
0x28, 0x00, 0x00, 0x00,  // 14: header size = 40
0x00, 0x00, 0x00, 0x00,  // 18: [xres]
0x00, 0x00, 0x00, 0x00,  // 22: [yres]
0x01, 0x00,              // 26: bit planes = 1
0x18, 0x00,              // 28: bits per pixel = 24
0x00, 0x00, 0x00, 0x00,  // 30: compression:  0=none, 1=8bit RLE, 2=4bit RLE, 3=RGB with mask
0x00, 0x00, 0x00, 0x00,  // 34: [image size]
0xC3, 0x0E, 0x00, 0x00,  // 38: Pixels per meter X (3779)
0xC3, 0x0E, 0x00, 0x00,  // 42: Pixels per meter Y
0x00, 0x00, 0x00, 0x00,  // 46: number of colors (pallette)
0x00, 0x00, 0x00, 0x00   // 50: important colors (pallette)
};

static bool BMP_Salva32To24( const char *name, unsigned char *buffer, int xres, int yres )
{
    HANDLE h = CreateFile(name, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, 0, NULL);
    if( h == INVALID_HANDLE_VALUE )
		return false;
#if 1
    *((int*)(bmpheader+ 2)) = xres*yres*3 + 54;
    *((int*)(bmpheader+18)) = xres;
    *((int*)(bmpheader+22)) = yres;
    *((int*)(bmpheader+34)) = xres*yres*3;

    DWORD written = 0;
    WriteFile(h, bmpheader, 54, &written, nullptr);

    unsigned unsigned char tmp[3*XRES];
    for( int j=0; j<yres; j++ )
    {
        for( int i=0; i<xres; i++ )
        {
            const unsigned char *ptr = buffer + 4*(xres*(yres-1-j)+i);
            tmp[3*i+0] = ptr[0];
            tmp[3*i+1] = ptr[1];
            tmp[3*i+2] = ptr[2];
        }
        WriteFile(h, tmp, 3*xres, &written, nullptr);
    }

#else
    const int N = 4;

    *((int*)(bmpheader+ 2)) = N*N*xres*yres*3 + 54;
    *((int*)(bmpheader+18)) = N*xres;
    *((int*)(bmpheader+22)) = N*yres;
    *((int*)(bmpheader+34)) = N*N*xres*yres*3;

    DWORD written = 0;
    WriteFile(h, bmpheader, 54, &written, nullptr);

    unsigned unsigned char tmp[3*XRES*N];
    for( int j=0; j<yres; j++ )
    {
        for( int i=0; i<xres; i++ )
        {
            const unsigned char *ptr = buffer + 4*(xres*(yres-1-j)+i);
            tmp[12*i+ 0] = ptr[0];
            tmp[12*i+ 1] = ptr[1];
            tmp[12*i+ 2] = ptr[2];
            tmp[12*i+ 3] = ptr[0];
            tmp[12*i+ 4] = ptr[1];
            tmp[12*i+ 5] = ptr[2];
            tmp[12*i+ 6] = ptr[0];
            tmp[12*i+ 7] = ptr[1];
            tmp[12*i+ 8] = ptr[2];
            tmp[12*i+ 9] = ptr[0];
            tmp[12*i+10] = ptr[1];
            tmp[12*i+11] = ptr[2];
        }
        WriteFile(h, tmp, 4*3*xres, &written, nullptr);
        WriteFile(h, tmp, 4*3*xres, &written, nullptr);
        WriteFile(h, tmp, 4*3*xres, &written, nullptr);
        WriteFile(h, tmp, 4*3*xres, &written, nullptr);
    }

#endif


	CloseHandle(h);

    return true;
}

extern "C" void entrypoint( void )
{
    MSG     msg;
    HWND    hWnd;
    DWORD   dwExStyle;
    DEVMODE dmScreenSettings;
    int     fullscreen;
    long    to = 0;
    RECT    rec = {0,0,XRES,YRES};
    int     done;


    fullscreen = 0;//(MessageBox( 0, "fullscreen?", TITULO, MB_YESNO|MB_ICONQUESTION)==IDYES );

    //--------------------------------------------------------

    if( fullscreen )
		{
        //memset(&dmScreenSettings,0,sizeof(DEVMODE));
        //for( int i=0; i<sizeof(DEVMODE); i++ ) { ((unsigned char*)&dmScreenSettings)[i] = 0; }

        dmScreenSettings.dmSize       = sizeof(DEVMODE);
        dmScreenSettings.dmFields     = DM_BITSPERPEL|DM_PELSWIDTH|DM_PELSHEIGHT;
        dmScreenSettings.dmBitsPerPel = 32;
        dmScreenSettings.dmPelsWidth  = XRES;
        dmScreenSettings.dmPelsHeight = YRES;

        if( ChangeDisplaySettings(&dmScreenSettings,CDS_FULLSCREEN)!=DISP_CHANGE_SUCCESSFUL)
            return;

        while( ShowCursor( 0 )>=0 );
        }
    else
		{
        dwStyle |= WS_CAPTION;
		}

    //--------------------------------------------------------

    AdjustWindowRect( &rec, dwStyle, 0 );

    hWnd = CreateWindowEx( WS_EX_TOPMOST, "STATIC", TITULO, dwStyle,
                           (GetSystemMetrics(SM_CXSCREEN)-rec.right+rec.left)>>1,
                           (GetSystemMetrics(SM_CYSCREEN)-rec.bottom+rec.top)>>1,
                           rec.right-rec.left, rec.bottom-rec.top,
                           0, 0,
                           GetModuleHandle(0),
                           0 );

    if( !hWnd ) return;

    SetForegroundWindow( hWnd );
    SetFocus( hWnd );

    //--------------------------------------------------------

    hDC = GetDC( hWnd );

    if( !hDC ) return;

    //--------------------------------------------------------

    pinta();

    intro_init();

    char fileName[] = "out/00000.bmp";
    const int kFps = 60;
    const int kDuration = 2*60 + 21;
    const int kNumFrames = kFps*kDuration;
    for( int i=0; i<kNumFrames; i++ )
    {
        PeekMessage(&msg,0,0,0,PM_REMOVE);

        const int time = (1000*i)/kFps;

        done = intro_do( buffer, time );

        pinta();

        
        fileName[8] = '0' + (i/1)%10;
        fileName[7] = '0' + (i/10)%10;
        fileName[6] = '0' + (i/100)%10;
        fileName[5] = '0' + (i/1000)%10;
        fileName[4] = '0' + (i/10000)%10;
        if( !BMP_Salva32To24( fileName, (unsigned char*)buffer, XRES, YRES ) ) break;

        if( msg.message==WM_KEYDOWN || done ) break;
    }

    //--------------------------------------------------------

    ReleaseDC( hWnd, hDC );
    DestroyWindow( hWnd );

    if( fullscreen )
		{
        ChangeDisplaySettings( 0, 0 );
        while( ShowCursor( 1 )<0 );
		}
}
