%include "main.inc"
%include "mmath.inc"

[GLOBAL CUADRIC_Interseca]
[GLOBAL CUADRIC_NormalAndText]

; iqiq
EXTERN normal
EXTERN zbuf
EXTERN aux0

[SEGMENT .data USE32]
R128        dd  128.0       ; ray

[SEGMENT .text USE32]

CUADRIC_SIZE equ (28+1)

%define A   (ebp+0)
%define E   (ebp+4)
%define H   (ebp+8)
%define D   (ebp+12)
%define G   (ebp+16)
%define I   (ebp+20)
%define J   (ebp+24)

%define Xd  (esi+0)
%define Yd  (esi+4)
%define Zd  (esi+8)

%define Xo  (edi+0)
%define Yo  (edi+4)
%define Zo  (edi+8)



;
; ebp : ptr a objeto
; esi : dir
; edi : pos
;
; si inter : cf=1 + t en pila
; no inter : cf=0 + pila vacia
;
CUADRIC_Interseca:

    pushad

    ;-------------------------
    ;a = A*Xd*Xd  +
    ;    E*Yd*Yd  +
    ;    H*Zd*Zd ;
    ;-------------------------

    xor     ecx, ecx
    mov     cl, 3
    @loopCalcA:                     
        fld     dword [esi+4*ecx-4]
        fmul    st0, st0
        fmul    dword [ebp+4*ecx-4]
    %if FAST==1
    dec     ecx
    jnz     @loopCalcA
    %else
    loop    @loopCalcA
    %endif
    ;
    faddp   st1
    faddp   st1


    ;-----------------------------------
    ;b'=   Xd*(A*Xo + D) +
    ;      Yd*(E*Yo + G) +
    ;      Zd*(H*Zo + I) ;
    ;-----------------------------------

    mov     cl, 3
    @loopCalcB:
        fld     dword [edi+4*ecx-4]
        fmul    dword [ebp+4*ecx-4]
        fadd    dword [ebp+12+4*ecx-4]
        fmul    dword [esi+4*ecx-4]
    %if FAST==1
    dec     ecx
    jnz     @loopCalcB
    %else
    loop    @loopCalcB
    %endif
    ;
    faddp   st1
    faddp   st1
    ;



    ;-----------------------------------
    ; c = cte en todo el frame... ¿precalc?
    ;-----------------------------------
    ; c = Xo*( A*Xo + 2.0*D) +
    ;     Yo*( E*Yo + 2.0*G) +
    ;     Zo*( H*Zo + 2.0*I) + J;
    ;-----------------------------------

    mov     cl, 3
    fld     dword [J]
    @loopCalcC:
        fld     dword [ebp+12-4+4*ecx]
        fadd    st0, st0
        fld     dword [edi-4+4*ecx]
        fmul    dword [ebp-4+4*ecx]
        faddp   st1
        fmul    dword [edi-4+4*ecx]
        faddp   st1
    %if FAST==1
    dec     ecx
    jnz     @loopCalcC
    %else
    loop    @loopCalcC
    %endif


    ;
    ; t = (-b - sqrt(b²-4ac)) / 2a
    ;
    ; si b' = b/2
    ;
    ; t = (-b' - sqrt(b'²-ac)) / a
    ;
                                                ; a b' c
    fmul    st0, st2                        ; a b' ca
    fld     st1                               ; a b' ac b'
    fmul    st0, st0                        ; a b' ac b'²
    fsubrp  st1                                 ; a b' b'²-ac
    
    ; interseca ?

    %if P6==1
    fldz                                          ; a b' b'²-ac 0.0
    fcomip   st0, st1                             ; a b' b'²-ac
    ;ftsti
    ja      @no_inter
    %else
    ftst                                        ; a b' b'²-ac
    fnstsw  ax
    sahf
    jb      @no_inter
    %endif

        fsqrt                       ; a  b' sqrt(b'²-ac)
        faddp st1                   ; a  b'+sqrt(b'²-ac)
        fchs                        ; a -b'-sqrt(b'²-ac)
        fdivrp st1                  ; (-b'-sqrt(b'²-ac))/a

        ; por delante de la cámara ?  t>0??
        %if P6==1
        fldz                                          ; a b' b'²-ac 0.0
        fcomip   st0, st1                             ; a b' b'²-ac
        ;ftsti
        ja      @no_inter2
        %else
        ftst                        ; t
        fnstsw  ax
        sahf
        jb      @no_inter2
        %endif

            ; y es la intersección más cercana ?
            %if P6==1
            fld     dword [zbuf]        ; t zbuf
            fcomip  st0, st1            ; t
            jb      @no_inter2
            %else
            fcom    dword [zbuf]        ; t
            fnstsw  ax
            sahf
            ja      @no_inter2
            %endif
                stc
                popad
                ret

    @no_inter:
    fstp    st0
    fstp    st0
    @no_inter2:
    fstp    st0

    clc
    popad
    ret


; ebx -> ptr o objeto
; machaca : ecx (lo deja a 0)
; deja en [normal] y en [aux0,aux1]
CUADRIC_NormalAndText:

    ; --- normal ---

    ; normal->x = A*inter->x + D;
    ; normal->y = E*inter->y + G;
    ; normal->z = H*inter->z + I;

    mov     esi, edi
    mov     edi, normal
    xor     ecx, ecx
    mov     cl, 3       ; mov ecx, 3
    @loopcalcnormal:
        fld     dword [ebx+4*ecx-4]
        fmul    dword [esi+4*ecx-4]
        fadd    dword [ebx+12+4*ecx-4]
        fstp    dword [edi+4*ecx-4]
    %if FAST==1
    dec     ecx
    jnz     @loopcalcnormal
    %else
    loop    @loopcalcnormal
    %endif
    
    call    MATH_Normalize

    ; --- text ----
    fld     dword [R128]      ; 128.0
    fld     st0               ; 128.0 128.0
    fmul    dword [edi+0]     ; 128.0 128.0·nx
    fistp   dword [aux0]      ; 128.0
    fmul    dword [edi+4]     ; 128.0·ny
    fistp   word [aux0+1]     ;

    ret


